
 {****************************************************************************}
 {                                                                            }
 {               FIBS Firebird-Interbase Backup Scheduler                     }
 {                                                                            }
 {                 Copyright (c) 2005-2006, Talat Dogan                       }
 {                                                                            }
 { This program is free software; you can redistribute it and/or modify it    }
 { under the terms of the GNU General Public License as published by the Free }
 { Software Foundation; either version 2 of the License, or (at your option)  }
 { any later version.                                                         }
 {                                                                            }
 { This program is distributed in the hope that it will be useful, but        }
 { WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY }
 { or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for}
 { more details.                                                              }
 {                                                                            }
 { You should have received a copy of the GNU General Public License along    }
 { with this program; if not, write to the Free Software Foundation, Inc.,    }
 { 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                      }
 {                                                                            }
 { Contact : dogantalat@yahoo.com
 {                                                                            }
 {****************************************************************************}


{ Backup Thread}

unit BackupUnit;

interface

uses
  Windows, Classes,SysUtils, MesajUnit;

type
  TBackup = class(TThread)
  private
    FSettings   : TFormatSettings;  // Thread-safe string formatting context.
    FPriority   : TThreadPriority;  // Scheduling priority of TBackup.
    FCompDegree     : string;    // GZip compression degree.
    FDeletedBU      : string;    // Filenames of the backup files deleted by "Backup Preserve Policy".
    FDeletedMR      : string;    // Filenames of the #1 Mirror files deleted by "Backup Preserve Policy".
    FDeletedMR2     : string;    // Filenames of the #2 Mirror files deleted by "Backup Preserve Policy".
    FDeletedMR3     : string;    // Filenames of the #3 Mirror files deleted by "Backup Preserve Policy".
    FOldestFileName : string;    // Filename of the oldest backup or mirror file
    FDeleteAll      : integer;   // 0=Delete only oldest file,  1= Delete all out-of-criteria backup files
    FPreserveUnit   : integer;   // Basic Backup Preserve styles
    FPreserveCount  : integer;   // Value of preserve style.
    FErrorInfo      : string;    // Basic error information
    FBackupNo       : string;    // Backup list number
    FZippedBackupFile  : string; // Fully qualified path of GZipped backup file
    FZippedMirrorFile  : string; // Fully qualified path of GZipped mirror file #1
    FZippedMirrorFile2 : string; // Fully qualified path of GZipped mirror file #2
    FZippedMirrorFile3 : string; // Fully qualified path of GZipped mirror file #3
    FTempDir           : string; // Temporary files directory
    FTempBackFileName  : string; // Fully qualified path of temporary file for backup
    FCompressBackup          : Boolean;   // 0= Do not create compressed backup, 1=Create compressed Backup
    FExternalFile   : string;    // Fully qualified path of external files to be runned
    FShowExternalWin: Boolean;   // if True, the output windows of the external file is shown.
    FUseParams      : Boolean;   // if True, the last backup file's name is passed to external files as a parameter
    FSmtpServer     : string;    // SMTP Server address
    FSendersMail    : string;    // Mail account
    FMailUserName   : string;    // User Name
    FMailPassword   : string;    // Password
    FMailTo         : string;    // Receiver email address' delimited by column (;)
    FAlarm          : TDateTime; // DateTime to Backup
    FConnectionType : dword;     // FTP Connection Mode. 0= Active Mode,  $08000000=Passive Mode
    FManAuto        : Boolean;   // 0=Manual Mode, 1=Automatic Mode
    FDoValidate     : Boolean;   // if True, database validation will be done
    FCreateTime     : string;    // Thread create time
    FFullGBakParams : string;    // Command line parameter to be used to run GBak.exe including output capture 
    FGBakParams     : string;    // Command line parameter to be used to run GBak.exe
    FGFixParams     : string;    // Command line parameter to be used to run GFix.exe
    FTempValFileName: string;    // Fully qualified path of temporary file for validation
    FDir            : String;    // FIBS Directory
    FTaskName       : string;    // Name of current backup task.
    FDatabaseFile   : string;    // Name of current backup task.
    FBackupFile     : string;    // Fully qualified filename of backup file
    FDoMirror       : Boolean;   // if True, backup file is copied onto Mirror #1
    FDoMirror2      : Boolean;   // if True, backup file is copied onto Mirror #2
    FDoMirror3      : Boolean;   // if True, backup file is copied onto Mirror #3
    FMirrorFile     : string;    // Fully qualified filename of Mirror File #1
    FMirrorFile2    : string;    // Fully qualified filename of Mirror File #2
    FMirrorFile3    : string;    // Fully qualified filename of Mirror File #3
    FLogFile        : string;    // Fully qualified filename of Log file for current task.
    FBackupOptions  : string;    // GBak switches
    FTempOrBackupFileSize : string; // Size of backup file in bytes
    PD              : TMesajForm;   // Message Dialog which shows the current process
  protected
    function StrToOEM (s:string) : string;
    function GetTempPath:String;
    function ConvertFileTime(LFT: TFileTime): TDateTime;
    function IsConnectedToInternet: Boolean;
    function GetTempBackupFileSize(const ATempFile:string):string;
    function WriteLog(ALogInfo: string):integer;
    function ProgramRunWait(CommandLine, CurrentDir: string; Wait: Boolean):Boolean;
    function ProgramRunWaitRedirected(TempFileName,CommandLine, DatabaseToVal,CurrentDir: string; var Info:string;Wait: Boolean):Boolean;
    function CreateTemporaryFilename(Const TempDirectory: String;var ATempFileName:string):Boolean;
    function DeleteOldest(ACurrentDir,ADirInfo:string):string;
    function DeleteLocalOldest(ACurrentDir,ADirInfo:string):string;
    function DeleteRemoteOldest(ACurrentDir,ADirInfo:string):string;
    function CreateEmail(Const EmailAddr, CC,Subject, Body,Attach: String):Boolean;
    function FtpUploadFileEX(InternetAccessType:DWORD;ProxyName,ProxyByPass,strHost, strUser, strPwd: string;InternetService,Port: Integer; LocalFilePath,RemoteDir,RemoteFilePath : string;TransferType,ConnectionType:DWORD;var AErrorMesaj:string ): Boolean;
    function MirrorFileEx(ASourceFile:string;var ADestFile,AFtpErrorMesaj:string;AConnectionType:dword):LongBool;
    function ExtractFilePathEx(const AFullPath: string): string;
    procedure ShowPD;            // Creates TMesajForm and passed handle to variable PD.
    procedure RefreshPD10;       // Process Messages
    procedure RefreshPD11;
    procedure RefreshPD12;
    procedure RefreshPD13;
    procedure RefreshPD20;
    procedure RefreshPD22;
    procedure RefreshPD23;
    procedure RefreshPD30;
    procedure RefreshPD31;
    procedure RefreshPD40;
    procedure HidePD;           // Hides and frees TMesajForm
    procedure Execute; override;
  public
     constructor Create(AAlarm:TDateTime;
                        AParams,VParams,ADir,ATaskName,ABackupOptions,
                        ADatabaseFile,ABackupFile,AMirrorFile,AMirrorFile2,
                        AMirrorFile3,ALogFile,ABackupNo,ACompDegree,
                        ASmtpServer,ASendersMail,AMailUserName,AMailPassword,
                        AMailTo,AExternalFile:string;
                        AUseParams,AShowExternalWin,ADoMirror,ADoMirror2,ADoMirror3,AManAuto,
                        ACompressBackup,ADoValidate:Boolean;
                        APCount,APUnit,ADeleteAll,AFtpConnType:integer;
                        ABackupPriority:TThreadPriority);
  end;


implementation


uses WinInet, ShellApi,StrUtils,DateUtils,ConstUnit,JRZip,SmtpSend;



//By Lance leonard
//http://www.techtricks.com/delphi/sendmail.php
function TBackup.ConvertFileTime(LFT: TFileTime): TDateTime;
var
  FDFT: DWORD;
  w:word;
begin
  Result := 0;
  if  FileTimeToDosDateTime(LFT, w, LongRec(FDFT).Lo)
    then Result := FileDateToDateTime(FDFT);
end;


function TBackup.ExtractFilePathEx(const AFullPath: string): string;
begin
   if AnsiLowerCase(AnsiLeftStr(trim(AFullPath),6))='ftp://' then
   begin
    result:=copy(AFullPath,1,LastDelimiter('/',AFullPath)-1);
   end else result:=ExtractFilePath(AFullPath);
end;


function TBackup.GetTempPath:String;
var
  Buffer : Array [0..MAX_PATH] of char;
begin
  Windows.GetTempPath(sizeof(Buffer),@buffer);
  Result:=Buffer;
end;

function TBackup.GetTempBackupFileSize(const ATempFile:string):string;
var
  hFile:TFileStream;
begin
  if FileExists(ATempFile) then
  begin
    hFile:=TFileStream.Create(ATempFile,fmOpenRead+fmShareDenyNone);
    try
      result:=FormatFloat('#,',hFile.size,FSettings);
    finally
      hFile.Free;
    end;
  end else result:=ATempFile+' is not exists !!';
end;


function TBackup.CreateEmail(Const EmailAddr, CC,Subject, Body,Attach: String):Boolean;
var
 TS:TStrings;
begin
  TS:=TStringList.Create;
  try
    TS.text:=body;
    Result:=SendToExTD(FSendersMail,EmailAddr,Subject, FSmtpServer,TS,FMailUserName,FMailPassword,FErrorInfo);
  finally
    TS.free;
  end;
end;


function TBackup.DeleteOldest(ACurrentDir,ADirInfo:string):string;
begin
  if AnsiLowerCase(AnsiLeftStr(trim(ACurrentDir),6))='ftp://'
    then result:=DeleteRemoteOldest(ACurrentDir,ADirInfo)
    else result:=DeleteLocalOldest(ACurrentDir,ADirInfo);
end;


function TBackup.DeleteRemoteOldest(ACurrentDir,ADirInfo:string):string;
begin
  Result:='No Backup File has been deleted in '+ADirInfo+' because of not supporting the remote file deletion on FTP Mirrors yet !!'#13#10'   ';
end;

{
 Below given functions does not works correctly with all ftp servers
 because it cannot index remote files by file date-time.


function TBackup.DeleteRemoteOldest(ACurrentDir,ADirInfo:string):string;
var
  hNet, hFTP,hsearch: HINTERNET;
  finddata: TWin32FindData;
  SearchRec : TSearchRec;

  RemFilePath:array[0..999] of Char;
  AErrorMesaj:string;
  TS:TStringList;
  s,t,Key,NameExt,Ext:string;
  i,PDot:integer;
  TestTime,OldestFileAge:TDateTime;
  OldestFileAgeStr:string;
  Deleted,ff:Boolean;
  DeletedFilesCount,TryCount:integer;
  DeletedFiles:string;
  Liste:string;

  ProxyName,ProxyByPass: string;
  FtpUserName,FtpPassword,FtpHost,FtpDir,FtpFile:string;
  p1,p2,p3,p4,p5:integer;
  ax,bx:extended;
begin
  Result := '';
  AErrorMesaj:='';
  ACurrentDir:=trim(ACurrentDir);
  ACurrentDir:=AnsiReplaceStr(ACurrentDir,'\','/');
  if RightStr(ACurrentDir,1)<>'/' then  ACurrentDir:=ACurrentDir+'/';
  p1:=PosEx(':',ACurrentDir,7);
  p2:=Pos('@',ACurrentDir);
  p3:=PosEx('/',ACurrentDir,7);
  p4:=LastDelimiter('/',ACurrentDir);
  if p4<5 then p4:=0;
  p5:=Length(ACurrentDir);
  FtpUserName:=copy(ACurrentDir,7,p1-6-1);
  FtpPassword:=copy(ACurrentDir,p1+1,p2-p1-1);
  FtpHost:=copy(ACurrentDir,p2+1,p3-p2-1);
  FtpDir:='/'+copy(ACurrentDir,p3+1,p4-p3-1)+'/';
//  FtpFile:=copy(ACurrentDir,p4+1,p5-p4);
  hNet := InternetOpen('Program_Name', // Agent
                        INTERNET_OPEN_TYPE_PRECONFIG, // AccessType
                        PChar(ProxyName),
                        PChar(ProxyBypass),
                        0);
  if hNet = nil then
  begin
    result:='   Unable to get access to WinInet.Dll'#13#10'   ';
    Exit;
  end;
  hFTP := InternetConnect(hNet, // Handle from InternetOpen
                          PChar(FtpHost),
                          INTERNET_DEFAULT_FTP_PORT,
                          PChar(FtpUserName),
                          PChar(FtpPassword),
                          INTERNET_SERVICE_FTP,
                          0,
                          0);
  if hFTP = nil then
  begin
    InternetCloseHandle(hNet);
    result:=Format('   Host "%s" is not available',[FtpHost],FSettings);
    Exit;
  end;

  if FtpSetCurrentDirectory(hFTP, PChar(FtpDir))=false then
  begin
    InternetCloseHandle(hNet);
    InternetCloseHandle(hFTP);
    Result:='   '+FtpDir+' is not available';
    exit;
  end;
    result:='';
    TryCount:=0;
    DeletedFiles:='';
    DeletedFilesCount:=0;
    NameExt:=ExtractFileName(FDatabaseFile);
    Ext:=ExtractFileExt(FDatabaseFile);
    Key:=FTaskName+'*';
    TS:=TStringList.Create;
    TS.Sorted:=true;
    try
      hSearch := FtpFindFirstFile(hFTP,PChar(Key),findData,0,0);
      if hSearch = nil then
      begin
      end else
      begin
        repeat
          s:=FloatToStr(ConvertFileTime(finddata.ftLastWriteTime),FSettings);
          PDot:=Pos(DecimalSeparator,s);
          if (PDot=0) then s:='0000000000'+s+DecimalSeparator+'0' else s:='0000000000'+s;
          PDot:=Pos(DecimalSeparator,s);
          s:=copy(s,PDot-10,20);
          TS.Add(s+' - '+finddata.cFileName);
        until not InternetFindNextFile(hSearch,@findData);
        InternetCloseHandle(hSearch);
      end;
      t:=TS.Text;
      TestTime:=Now;
      if (FDeleteAll=0) then
      begin
        //Sadece en eski tarihli dosya siliniyor
        Deleted:=false;
        OldestFileAge:=StrToFloat((copy(TS[0],1,Pos(' - ',TS[0])-1)),FSettings); // En eski dosya
        OldestFileAgeStr:=DateTimeToStr(OldestFileAge,FSettings);
        FOldestFileName:=RightStr(TS[0],Length(TS[0])-Pos(' - ',TS[0])-3+1);
        if (FPreserveUnit=0)then  if (TS.Count>FPreserveCount) then
        begin
          inc(TryCount);
          Deleted:=FtpDeleteFile(hFTP,PChar(FtpDir+FOldestFileName)); //Son ..kopya
        end;
        if (FPreserveUnit=1)then  if (OldestFileAge<(TestTime-FPreserveCount/24))then
        begin
          inc(TryCount);
          Deleted:=FtpDeleteFile(hFTP,PChar(FtpDir+FOldestFileName));
        end;
        if (FPreserveUnit=2)then  if (OldestFileAge<(TestTime-FPreserveCount))   then
        begin
          inc(TryCount);
          Deleted:=FtpDeleteFile(hFTP,PChar(FtpDir+FOldestFileName));
        end;
        if (FPreserveUnit=3)then  if (OldestFileAge<(TestTime-FPreserveCount*30))then
        begin
          inc(TryCount);
          Deleted:=FtpDeleteFile(hFTP,PChar(FtpDir+FOldestFileName));
        end;
        if Deleted then inc(DeletedFilesCount);
        if TryCount>0 then
        begin
          if (DeletedFilesCount=TryCount)
             then Result:='The oldest backup file in '+ADirInfo+' ('+FOldestFileName+' '+OldestFileAgeStr+') has been deleted.'#13#10'   '+AErrorMesaj+#13#10'   '
             else Result:='ERROR !! The oldest backup file in '+ADirInfo+' ('+FOldestFileName+' '+OldestFileAgeStr+') couldn''t deleted.'#13#10'   '+AErrorMesaj+#13#10'   ';
        end else Result:='There is no out-of-criteria backup file to delete in '+ADirInfo+''#13#10'   ';
      end else
      begin
        //Btn krtere uymayanlar siliniyor
        for i:=0 to TS.Count-1 do
        begin
          Deleted:=false;
          OldestFileAge:=StrToFloat((copy(TS[i],1,Pos(' - ',TS[i])-1)),FSettings); // En eski dosya
          OldestFileAgeStr:=DateTimeToStr(OldestFileAge,FSettings);
          FOldestFileName:=RightStr(TS[i],Length(TS[i])-Pos(' - ',TS[i])-3+1);
          if (FPreserveUnit=0)then  if (TS.Count>FPreserveCount) then
          begin
            inc(TryCount);
            Deleted:=FtpDeleteFile(hFTP,PChar(FtpDir+FOldestFileName)); //Son ..kopya
          end;
          ax:=TestTime-FPreserveCount/24;
          bx:=OldestFileAge-ax;
          if (FPreserveUnit=1)then  if (OldestFileAge<(TestTime-FPreserveCount/24)) then
          begin
            inc(TryCount);
            Deleted:=FtpDeleteFile(hFTP,PChar(FtpDir+FOldestFileName));
          end;
          if (FPreserveUnit=2)then  if (OldestFileAge<(TestTime-FPreserveCount)) then
          begin
            inc(TryCount);
            Deleted:=FtpDeleteFile(hFTP,PChar(FtpDir+FOldestFileName));
          end;
          if (FPreserveUnit=3)then  if (OldestFileAge<(TestTime-FPreserveCount*30)) then
          begin
            inc(TryCount);
            Deleted:=FtpDeleteFile(hFTP,PChar(FtpDir+FOldestFileName));
          end;
          if Deleted then
          begin
            DeletedFiles:=DeletedFiles+FOldestFileName+' '+OldestFileAgeStr+', ';
            inc(DeletedFilesCount);
          end;
        end;
        if TryCount>0 then
        begin
          if (DeletedFilesCount=0) then Result:='ERROR !! None of Out-of-criteria backup files in '+ADirInfo+' has been deleted.'#13#10'   '+AErrorMesaj+#13#10'   '
          else if DeletedFilesCount=TryCount then Result:='All Out-of-criteria backup files ('+DeletedFiles+') in the '+ADirInfo+' has been deleted.'#13#10'   '+AErrorMesaj+#13#10'   '
          else Result:='ERROR !! Only '+IntToStr(DeletedFilesCount)+' of '+IntToStr(TryCount)+' Out-of-criteria backup files ('+DeletedFiles+') in '+ADirInfo+' has been deleted.'#13#10'   '+AErrorMesaj+#13#10'   ';
        end else Result:='There is no out-of-criteria backup file to delete in '+ADirInfo+''#13#10'   ';
      end;
    finally
      TS.Free;
    end;
  InternetCloseHandle(hFTP);
  InternetCloseHandle(hNet);
end;
}


function TBackup.DeleteLocalOldest(ACurrentDir,ADirInfo:string):string;
var
  SearchRec : TSearchRec;
  TS:TStringList;
  t,s,Key,NameExt,Ext:string;
  i,PDot:integer;
  TestTime,OldestFileAge:TDateTime;
  OldestFileAgeStr:string;
  Deleted:Boolean;
  DeletedFilesCount,TryCount:integer;
  DeletedFiles:string;
begin
  result:='';
  TryCount:=0;
  DeletedFiles:='';
  DeletedFilesCount:=0;
  NameExt:=ExtractFileName(FDatabaseFile);
  Ext:=ExtractFileExt(FDatabaseFile);
  if (ACurrentDir[Length(ACurrentDir)] <> '\') then ACurrentDir := ACurrentDir + '\';
  Key:=ACurrentDir+FTaskName+'*';
  TS:=TStringList.Create;
  TS.Sorted:=true;
  try
    if FindFirst(Key, faAnyFile, SearchRec) = 0 then
    begin
      repeat
       s:=FloatToStr(FileDateToDateTime(SearchRec.Time),FSettings);
       PDot:=Pos(DecimalSeparator,s);
       if (PDot=0) then s:='0000000000'+s+DecimalSeparator+'0' else s:='0000000000'+s;
       PDot:=Pos(DecimalSeparator,s);
       s:=copy(s,PDot-10,20);
       TS.Add(s+' - '+SearchRec.Name);
      until FindNext(SearchRec) <> 0;
      FindClose(SearchRec);
      t:=TS.Text;
      TestTime:=Now;
      if (FDeleteAll=0) then
      begin
        //Delete only the oldest file
        Deleted:=false;
        OldestFileAge:=StrToFloat((copy(TS[0],1,Pos(' - ',TS[0])-1)),FSettings);       // En eski dosya
        OldestFileAgeStr:=DateTimeToStr(OldestFileAge,FSettings);
        FOldestFileName:=RightStr(TS[0],Length(TS[0])-Pos(' - ',TS[0])-3+1);
        if (FPreserveUnit=0)then  if ((TS.Count>FPreserveCount)) then
        begin
          inc(TryCount);
          Deleted:=DeleteFile(PChar(ACurrentDir+FOldestFileName)); //Son ..kopya
        end;
        if (FPreserveUnit=1)then  if (OldestFileAge<(TestTime-FPreserveCount/24)) then
        begin
          inc(TryCount);
          Deleted:=DeleteFile(PChar(ACurrentDir+FOldestFileName));
        end;
        if (FPreserveUnit=2)then  if (OldestFileAge<(TestTime-FPreserveCount)) then
        begin
          inc(TryCount);
          Deleted:=DeleteFile(PChar(ACurrentDir+FOldestFileName));
        end;
        if (FPreserveUnit=3)then  if (OldestFileAge<TestTime-FPreserveCount*30) then
        begin
          inc(TryCount);
          Deleted:=DeleteFile(PChar(ACurrentDir+FOldestFileName));
        end;
        if Deleted then inc(DeletedFilesCount);
        if TryCount>0 then
        begin
          if (DeletedFilesCount=TryCount)
             then Result:='The oldest backup file in '+ADirInfo+' ('+FOldestFileName+' '+OldestFileAgeStr+') has been deleted.'#13#10
             else Result:='ERROR !! The oldest backup file in '+ADirInfo+' ('+FOldestFileName+' '+OldestFileAgeStr+') couldn''t deleted.'#13#10
        end else Result:='There is no out-of-criteria backup file to delete in '+ADirInfo+''#13#10;
      end else
      begin
        // Delete all out-of-criteriaa files
        for i:=0 to TS.Count-1 do
        begin
          Deleted:=false;
          OldestFileAge:=StrToFloat((copy(TS[0],1,Pos(' - ',TS[0])-1)),FSettings); // En eski dosya
          OldestFileAgeStr:=DateTimeToStr(OldestFileAge,FSettings);
          FOldestFileName:=RightStr(TS[0],Length(TS[0])-Pos(' - ',TS[0])-3+1);
          if (FPreserveUnit=0)then  if (TS.Count>FPreserveCount) then
          begin
            inc(TryCount);
            Deleted:=DeleteFile(PChar(ACurrentDir+FOldestFileName)); //Son ..kopya
          end;
          if (FPreserveUnit=1)then  if (OldestFileAge<(TestTime-FPreserveCount/24)) then
          begin
            inc(TryCount);
            Deleted:=DeleteFile(PChar(ACurrentDir+FOldestFileName));
          end;
          if (FPreserveUnit=2)then  if (OldestFileAge<(TestTime-FPreserveCount)) then
          begin
            inc(TryCount);
            Deleted:=DeleteFile(PChar(ACurrentDir+FOldestFileName));
          end;
          if (FPreserveUnit=3)then  if (OldestFileAge<TestTime-FPreserveCount*30) then
          begin
            inc(TryCount);
            Deleted:=DeleteFile(PChar(ACurrentDir+FOldestFileName));
          end;
          if Deleted then
          begin
             TS.Delete(0);
             DeletedFiles:=DeletedFiles+FOldestFileName+' '+OldestFileAgeStr+', ';
             inc(DeletedFilesCount);
          end;
        end;
        if TryCount>0 then
        begin
          if (DeletedFilesCount=0) then Result:='ERROR !! None of Out-of-criteria backup files in '+ADirInfo+' has been deleted.'#13#10
          else if DeletedFilesCount=TryCount then Result:='All Out-of-criteria backup files ('+DeletedFiles+') in the '+ADirInfo+' has been deleted.'#13#10
          else Result:='ERROR !! Only '+IntToStr(DeletedFilesCount)+' of '+IntToStr(TryCount)+' Out-of-criteria backup files ('+DeletedFiles+') in '+ADirInfo+' has been deleted.'#13#10;
        end else Result:='There is no out-of-criteria backup file to delete in '+ADirInfo+''#13#10;
      end;
    end;
  finally
    TS.Free;
  end;
end;



function TBackUp.CreateTemporaryFilename(const TempDirectory: string;var ATempFileName:string):Boolean;
Const
  TMP_FILE_PREFIX = '~'; {Must be 3 characters or less}
var
  TemporaryFileName: array [0..MAX_PATH-1] of char;
Begin
  result:=false;
  ATempFileName:='';
  if GetTempFileName(PChar(TempDirectory), TMP_FILE_PREFIX, 0, TemporaryFileName) >0 then
  begin
     result:=true;
     ATempFileName:=TemporaryFileName;
  end;
end;


function TBackUp.IsConnectedToInternet: Boolean;
var
  dwConnectionTypes: DWORD;
begin
  dwConnectionTypes := INTERNET_CONNECTION_MODEM +
                       INTERNET_CONNECTION_LAN +
                       INTERNET_CONNECTION_PROXY;
  Result := InternetGetConnectedState(@dwConnectionTypes, 0);
end;

//Author Talat Dogan. I've inspirated Thomas Stutz.
function TBackUp.FtpUploadFileEX(InternetAccessType:DWORD;ProxyName,ProxyByPass,strHost, strUser, strPwd: string;InternetService,Port: Integer; LocalFilePath,RemoteDir,RemoteFilePath : string;TransferType,ConnectionType:DWORD;var AErrorMesaj:string ): Boolean;
var
  hNet, hFTP: HINTERNET;
  PutFile: Boolean;
begin
  Result := False;
//  Open an internet session
  hNet := InternetOpen('Program_Name', // Agent
                        InternetAccessType, // AccessType
                        PChar(ProxyName),
                        PChar(ProxyBypass),
                        0); // or INTERNET_FLAG_ASYNC / INTERNET_FLAG_OFFLINE
// Agent contains the name of the application or entity calling the Internet functions
// See if connection handle is valid
  if hNet = nil then
  begin
    AErrorMesaj:='   Unable to get access to WinInet.Dll';
    Exit;
  end;
  { Connect to the FTP Server }
  hFTP := InternetConnect(hNet, // Handle from InternetOpen
                          PChar(strHost), // FTP server
                          port, // (INTERNET_DEFAULT_FTP_PORT),
                          PChar(StrUser), // username
                          PChar(strPwd),  // password
                          InternetService, //INTERNET_SERVICE_FTP = 1; INTERNET_SERVICE_GOPHER = 2; INTERNET_SERVICE_HTTP = 3;
                          ConnectionType, // flag: 0 or INTERNET_FLAG_PASSIVE
                          0);// User defined number for callback
  if hFTP = nil then
  begin
    InternetCloseHandle(hNet);
    AErrorMesaj:=Format('   Host "%s" is not available',[strHost],FSettings);
    Exit;
  end;

  if (RemoteDir<>'') then
  begin
    if FtpSetCurrentDirectory(hFTP, PChar(RemoteDir))=false then
    begin
      if FtpCreateDirectory(hFTP, PChar(RemoteDir))=false then
      begin
        InternetCloseHandle(hFTP);
        InternetCloseHandle(hNet);
        AErrorMesaj:=Format('   Cannot Create directory to %s.',[RemoteDir],FSettings);
        Exit;
      end else FtpSetCurrentDirectory(hFTP, PChar(RemoteDir));
    end;
  end;
  { Upload file }
  PutFile := FtpPutFile(hFTP, // Handle to the ftp session
                       PChar(LocalFilePath), // filename
                       PChar(RemoteFilePath), // filename
                       TransferType, // dwFlags
                       0); // This is the context used for callbacks.
  if PutFile = False then
  begin
    InternetCloseHandle(hFTP);
    InternetCloseHandle(hNet);
     AErrorMesaj:=Format('   Cannot upload backup file ('+RemoteFilePath+') to host %s.',[strHost],FSettings);
    Exit;
  end;
  InternetCloseHandle(hFTP);
  InternetCloseHandle(hNet);
  Result := True;
end;




function TBackUp.MirrorFileEx(ASourceFile:string;var ADestFile,AFtpErrorMesaj:string;AConnectionType:dword):LongBool;
var
 FtpUserName,FtpPassword,FtpHost,FtpDir,FtpFile:string;
 FtpPort,FColPos,FAtPos,FDirPos,LLDirPos,LenPath:integer;
// LLColPos,LLAtPos :integer;
begin
  AFtpErrorMesaj:='';
  ADestFile:=trim(ADestFile);
  if AnsiLowerCase(AnsiLeftStr(ADestFile,6))='ftp://' then
  begin
    ADestFile:=AnsiReplaceStr(ADestFile,'\','/');
    FColPos:=PosEx(':',ADestFile,7);
//  LLColPos:=LastDelimiter(':',ADestFile);
    FAtPos:=Pos('@',ADestFile);
//  LLAtPos:=LastDelimiter('@',ADestFile);
    FDirPos:=PosEx('/',ADestFile,7);
    LLDirPos:=LastDelimiter('/',ADestFile);
    LenPath:=Length(ADestFile);

    FtpUserName:=copy(ADestFile,7,FColPos-6-1);
    FtpPassword:=copy(ADestFile,FColPos+1,FAtPos-FColPos-1);
    FtpHost:=copy(ADestFile,FAtPos+1,FDirPos-FAtPos-1);
    FtpDir:=copy(ADestFile,FDirPos+1,LLDirPos-FDirPos-1);
    FtpFile:=copy(ADestFile,LLDirPos+1,LenPath-LLDirPos);
    FtpPort:=21;

    result:=FtpUploadFileEx(INTERNET_OPEN_TYPE_PRECONFIG,
                     '','',
                     FtpHost,FtpUserName,FtpPassword,
//                     INTERNET_SERVICE_FTP,21,
                     INTERNET_SERVICE_FTP,FtpPort,
                     ASourceFile,FtpDir,FtpFile,
                     FTP_TRANSFER_TYPE_BINARY,AConnectionType,AFtpErrorMesaj);
    if AFtpErrorMesaj<>'' then AFtpErrorMesaj:=AFtpErrorMesaj+#13#10;
  end else result:=CopyFile(PChar(ASourceFile),PChar(ADestFile),false);
end;

function TBackUp.WriteLog(ALogInfo: string):integer;
var
 F:TextFile;
 LogDir: string;
begin
  result:=0;
  LogDir:=ExtractFileDir(FLogFile);
  try
    AssignFile(F,FLogFile);
    FileMode:=fmOpenWrite+fmShareDenyWrite;
    if FileExists(FLogFile)=false then Rewrite(F);
    Append(f);
    write(F, ALogInfo);
    Flush(f);
    CloseFile(f);
  except
  end;
end;


function TBackUp.ProgramRunWait(CommandLine, CurrentDir: string; Wait: Boolean):Boolean;
var
  StartupInfo: TStartupInfo;
  ProcessInfo: TProcessInformation;
  pCurrentDir: pChar;
  ExitCode: DWORD;
  CreResult:LongBool;
begin
  result:=false;
  //Fill the record with 0's so you don't pass in random bollox to the API call
  FillChar (StartupInfo, SizeOf(StartupInfo), 0);
  //and tell it the size of itself
  StartupInfo.cb := SizeOf(StartupInfo);
  StartupInfo.dwFlags:=STARTF_USESHOWWINDOW;
	StartupInfo.wShowWindow:= SW_HIDE;  //setup default dir

  if CurrentDir <> '' then pCurrentDir := pChar(CurrentDir) else pCurrentDir := nil;
  //Now do the API call
  CreResult:=CreateProcess(nil, pChar(CommandLine), nil, nil, False, 0, nil, pCurrentDir, StartupInfo, ProcessInfo);
  if CreResult=true then
  //lpApplication??
  begin
    try
      if Wait then
      begin
        case WaitForSingleObject(ProcessInfo.hProcess, {12000)//test}INFINITE) of
          WAIT_ABANDONED:;
//            MessageBox(0, PChar('WAIT_ABANDONED'), PChar('Process returned'), 0);
          WAIT_OBJECT_0:;
//            MessageBox(0, PChar('WAIT_OBJECT_0'), PChar('Process returned'), 0);
          WAIT_TIMEOUT:;
//            MessageBox(0, PChar('WAIT_TIMEOUT'), PChar('Process returned'), 0);
          WAIT_FAILED:;
//            MessageBox(0, PChar('WAIT_FAILED'), PChar('Process returned'), 0);
        end;
        GetExitCodeProcess(ProcessInfo.hProcess, ExitCode);
        Result := (ExitCode = 0);
      end;
    finally
      CloseHandle(ProcessInfo.hThread);
      CloseHandle(ProcessInfo.hProcess);
    end;
  end else Result:=false;
end;

function TBackup.ProgramRunWaitRedirected(TempFileName,CommandLine, DatabaseToVal,CurrentDir: string; var Info:string;Wait: Boolean):Boolean;
var
  StartupInfo: TStartupInfo;
  ProcessInfo: TProcessInformation;
  pCurrentDir: pChar;
  ExitCode: DWORD;
  CreResult:LongBool;

  tmpName: string;
  tmp: Windows.THandle;
  tmpSec: TSecurityAttributes;
  res: TStringList;
begin
  result:=false;
//  tmpName := 'Test.tmp';
  tmpName := TempFileName;
  FillChar(tmpSec, SizeOf(tmpSec), #0);
  tmpSec.nLength := SizeOf(tmpSec);
  tmpSec.bInheritHandle := True;
  tmp := Windows.CreateFile(PChar(tmpName),
         Generic_Write, File_Share_Write,
         @tmpSec, Create_Always, File_Attribute_Normal, 0);

  //Fill the record with 0's so you don't pass in random bollox to the API call
  FillChar (StartupInfo, SizeOf(StartupInfo), 0);
  //and tell it the size of itself
  StartupInfo.cb := SizeOf(StartupInfo);
  StartupInfo.hStdError  := tmp;
  StartupInfo.dwFlags     := STARTF_USESTDHANDLES or STARTF_USESHOWWINDOW;
	StartupInfo.wShowWindow:= SW_HIDE;  //setup default dir

  if CurrentDir <> '' then pCurrentDir := pChar(CurrentDir) else pCurrentDir := nil;
  //Now do the API call
  CreResult:=CreateProcess(nil, pChar(CommandLine+' '+DatabaseToVal), nil, nil, true, 0, nil, pCurrentDir, StartupInfo, ProcessInfo);
  if CreResult=true then
  //lpApplication??
  begin
    try
      if Wait then
      begin
        case WaitForSingleObject(ProcessInfo.hProcess, {12000)//test}INFINITE) of
          WAIT_ABANDONED:;
//            MessageBox(0, PChar('WAIT_ABANDONED'), PChar('Process returned'), 0);
          WAIT_OBJECT_0:;
//            MessageBox(0, PChar('WAIT_OBJECT_0'), PChar('Process returned'), 0);
          WAIT_TIMEOUT:;
//            MessageBox(0, PChar('WAIT_TIMEOUT'), PChar('Process returned'), 0);
          WAIT_FAILED:;
//            MessageBox(0, PChar('WAIT_FAILED'), PChar('Process returned'), 0);
        end;
        GetExitCodeProcess(ProcessInfo.hProcess, ExitCode);
        Result := (ExitCode = 0);
      end;
    finally
      CloseHandle(ProcessInfo.hThread);
      CloseHandle(ProcessInfo.hProcess);
      CloseHandle(tmp);
      info:='';
      res := TStringList.Create;
      try
        res.LoadFromFile(tmpName);
        Info:=res.Text;
      finally
        res.Free;
      end;
      Windows.DeleteFile(PChar(tmpName));
    end;
  end else
  begin
    Info:=CommandLine+' couldn''t been executed ..';
    Result:=false;
  end;
end;

constructor TBackUp.Create(AAlarm:TDateTime;
                           AParams,VParams,ADir,ATaskName,ABackupOptions,
                           ADatabaseFile,ABackupFile,AMirrorFile,AMirrorFile2,
                           AMirrorFile3,ALogFile,ABackupNo,ACompDegree,
                           ASmtpServer,ASendersMail,AMailUserName,AMailPassword,
                           AMailTo,AExternalFile:string;
                           AUseParams,AShowExternalWin,ADoMirror,ADoMirror2,
                           ADoMirror3,AManAuto,ACompressBackup,ADoValidate:Boolean;
                           APCount,APUnit,ADeleteAll,AFtpConnType:integer;
                           ABackupPriority:TThreadPriority);
begin
  GetLocaleFormatSettings(SysLocale.DefaultLCID,FSettings);
  FPriority:=ABackupPriority;
  FCompDegree:=ACompDegree;
  FDeleteAll:=ADeleteAll;
  FPreserveCount:=APCount;
  FPreserveUnit:=APUnit;
  FBackupNo:=ABackupNo;
  FCompressBackup:=ACompressBackup;
  FExternalFile:=AExternalFile;
  FUseParams:=AUseParams;
  FShowExternalWin:=AShowExternalWin;
  FSmtpServer:=ASmtpServer;
  FSendersMail:=ASendersMail;
  FMailUserName:=AMailUserName;
  FMailPassword:=AMailPassword;
  FMailTo:=AMailTo;
  FDoValidate:=ADoValidate;
  FAlarm:=AAlarm;
  FCreateTime:=DateTimeToStr(Now,FSettings);
  FDoMirror:=ADoMirror;
  FDoMirror2:=ADoMirror2;
  FDoMirror3:=ADoMirror3;
  if AFtpConnType=0
     then FConnectionType:=0 //Active Mode
     else FConnectionType:=INTERNET_FLAG_PASSIVE; // Passive Mode

  FManAuto:=AManAuto;
  FGBakParams:=AParams+' -y '+GetTempPath+'~fibs_gbakout_'+FBackupNo+'.tmp';
  FGFixParams:=VParams;
  FTempValFileName:=GetTempPath+'~fibs_gfixout_'+FBackupNo+'.tmp';
  FDir:=ADir;
  FTaskName:=ATaskName;

  FDatabaseFile:=ADatabaseFile;
//Beta-13   if Pos(' ',ADatabaseFile)>0 then FDatabaseFile:='"'+ADatabaseFile+'"';
  //Dikkat sadece yukar satr Ver. 1.0.4

  FErrorInfo:='';

  FBackupFile:=ABackupFile;
//Beta-13  if Pos(' ',ABackupFile)>0 then FBackupFile:='"'+ABackupFile+'"';

  FMirrorFile:=AMirrorFile;
  FMirrorFile2:=AMirrorFile2;
  FMirrorFile3:=AMirrorFile3;
  FLogFile:=ALogFile;
  FBackupOptions:=ABackupOptions;
  FreeOnTerminate := True;
  inherited Create(False);
  Priority := FPriority;
end;

procedure TBackup.ShowPD;
begin
  PD:=ShowProsesDlg('Back up task "'+FTaskName+'" is executing'#13#10'BackUp is preparing..','c','');
end;
procedure TBackup.RefreshPD10;
begin
  RefreshProsesDlg('Back up task "'+FTaskName+'" is executing'#13#10'BackUp File is GZipping..','c',PD);
end;
procedure TBackup.RefreshPD11;
begin
//  RefreshProsesDlg('Back up task "'+FTaskName+'" is executing'#13#10'BackUp File is GZipping..'+#13#10'MakeZip OK','c',PD);
  RefreshProsesDlg('Back up task "'+FTaskName+'" is executing'#13#10'Database is Validating..','c',PD);
end;
procedure TBackup.RefreshPD12;
begin
  RefreshProsesDlg('Back up task "'+FTaskName+'" is executing'#13#10'BackUp is preparing..','c',PD);
end;
procedure TBackup.RefreshPD13;
begin
//  RefreshProsesDlg('Back up task "'+FTaskName+'" is executing'#13#10'BackUp File is GZipping..'+#13#10'CloseZip OK','c',PD);
  RefreshProsesDlg('Back up task "'+FTaskName+'" is executing'#13#10'Sending EMail Notification..'+#13#10'','c',PD);
end;
procedure TBackup.RefreshPD20;
begin
  RefreshProsesDlg('Back up task "'+FTaskName+'" is executing'#13#10'BackUp File is mirroring to Mirror #1..','c',PD);
end;
procedure TBackup.RefreshPD22;
begin
  RefreshProsesDlg('Back up task "'+FTaskName+'" is executing'#13#10'BackUp File is mirroring to Mirror #2..','c',PD);
end;
procedure TBackup.RefreshPD23;
begin
  RefreshProsesDlg('Back up task "'+FTaskName+'" is executing'#13#10'BackUp File is mirroring to Mirror #3..','c',PD);
end;
procedure TBackup.RefreshPD30;
begin
  RefreshProsesDlg('Back up task "'+FTaskName+'" is executing'#13#10'Out-of-Criteria files is deleting in backup dir and mirrors dir(s)..','c',PD);
end;
procedure TBackup.RefreshPD31;
begin
  RefreshProsesDlg('Back up task "'+FTaskName+'" is executing'#13#10+FExternalFile+'is starting to be done extra task.','c',PD);
end;
procedure TBackup.RefreshPD40;
begin
  RefreshProsesDlg('Back up task "'+FTaskName+'" is executing'#13#10'Writing in LOG..','c',PD);
end;
procedure TBackup.HidePD;
begin
  HideProsesDlg(PD);
end;

procedure TBackup.Execute;
var
  AStartTime,AEndTime:TDateTime;
  BackupInfoStr:string;
  MailBody,BUPath,MirrorPath,MirrorPath2,MirrorPath3,MMail,MDel,MBack,MFtpCopy,MFtpCopy2,MFtpCopy3,MCopy,MCopy2,MCopy3,MAuto:string;
  ValidateOK,EMailOK,TempOK,ZipOK,BCreOK,BackOK,CopyOK,CopyOK2,CopyOK3: LongBool;
  GOut,GFixOut,ss,BackupExt,AEndTimeStr:string;
  i,CompLevel:integer;
  MirrorIsFtp,Mirror2IsFtp,Mirror3IsFtp:Boolean;
  BatchFileOk:Boolean;
  ParamsToBatch, MBatch:string;
  TX:TStrings;
  FtpModeInfoA,FtpModeInfoB:string;
  ExternalResultCode:cardinal;
begin
  Synchronize(ShowPD);
  try
//    try
    AStartTime:=Now;
    FTempDir:=GetTempPath;
    TempOK:=false;
    FErrorInfo:='';
    MailBody:='Below listed errors has been encountered at the backup of Task '+FTaskName+' BackupNo :'+FBackupNo; 
//****** Validating Routines  ***************
    ValidateOK:=false;
    if FDoValidate then
    begin
      Synchronize(RefreshPD11);
      ProgramRunWaitRedirected(FTempValFileName,FGFixParams,'"'+FDatabaseFile+'"','',GFixOut,true);
      if GFixOut='' then
      begin
        ValidateOK:=true;
        GFixOut:='Database was validated (before backup) by using gfix with -v -n switches. No validation error was found..'#13#10;
      end else
      begin
        TX := TStringList.Create;
        try
          TX.text:=GFixOut;
          TX.Insert(0,'--------------------------------------------------------------------------------------   GFix.exe Output   ----------------------------------------------------------------------------------');
          TX.Add('---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------');
          for i:=0 to TX.Count-1 do TX[i]:='   '+TX[i];
          GFixOut:=TX.text;
        finally
          TX.Free;
        end;
        MailBody:=MailBody+#13#10#13#10+FTaskName+' has encountered database Validating Error !!';
        ValidateOK:=false;
      end;
    end;
//****** ***********************************
    Synchronize(RefreshPD12);
    ss:=FBackupFile;
    BackupExt:=ExtractFileExt(FBackupFile);
    FBackupFile:=copy(FBackupFile,1,Pos(BackupExt,FBackupFile)-1)+'-'+FBackupNo+BackupExt;
    BackupExt:=ExtractFileExt(FBackupFile);
    FMirrorFile:=copy(FMirrorFile,1,Pos(BackupExt,FMirrorFile)-1)+'-'+FBackupNo+BackupExt;
    FMirrorFile2:=copy(FMirrorFile2,1,Pos(BackupExt,FMirrorFile2)-1)+'-'+FBackupNo+BackupExt;
    FMirrorFile3:=copy(FMirrorFile3,1,Pos(BackupExt,FMirrorFile3)-1)+'-'+FBackupNo+BackupExt;

    BUPath:=ExtractFilePath(FBackupFile);
//  FZippedBackupFile:=ChangeFileExt(FBackupFile,'.ZIP');
//  FZippedMirrorFile:=ChangeFileExt(FMirrorFile,'.ZIP');
//  FZippedBackupFile:=ChangeFileExt(FBackupFile,'.GZ');
//  FZippedMirrorFile:=ChangeFileExt(FMirrorFile,'.GZ');

    if DirectoryExists(BUPath)=false then BCreOK:=false else BCreOK:=true;
    if BCreOK=true then
    begin
      if FCompressBackup then
      begin
        if DirectoryExists(FTempDir)=false then
        begin
          if CreateDir(FTempDir) then TempOK:=CreateTemporaryFilename(FTempDir,FTempBackFileName);
        end else TempOK:=CreateTemporaryFilename(FTempDir,FTempBackFileName);
        //Yukardaki nansa dikkat TempDir yaratlamazsa TempOk:=false oluyor..

        if TempOK then //Eer ZIP isteniyorsa Temp yaratlm olmal
        begin
          BackOK:=ProgramRunWait(FGBakParams+' "'+FDatabaseFile+'" '+FTempBackFileName, FDir,true);
          FFullGBakParams:=FGBakParams+' '+FDatabaseFile+' '+FTempBackFileName;
          FTempOrBackupFileSize:=GetTempBackupFileSize(FTempBackFileName);
          if BackOK then
          begin
            //Sadece BackOK=true ise FileSize kontrol edilmeli.
            // Yoksa yarm kalm bir backup 'n boyutu 0'dan farkl olduu iin BackOK
            // olmas iten bile deil !!!
            if FTempOrBackupFileSize>'0' then BackOK:=true else BackOK:=false;
          end;
        end else BackOK:=false;
      end else
      begin
        BackOK:=ProgramRunWait(FGBakParams+' "'+FDatabaseFile+'" '+' "'+FBackupFile+'" ', FDir,true);
        FFullGBakParams:=FGBakParams+' '+FDatabaseFile+' '+' '+FBackupFile+' ';
        FTempOrBackupFileSize:=GetTempBackupFileSize(FBackupFile);
        if BackOK then
        begin
          //Sadece BackOK=true ise FileSize kontrol edilmeli.
          // Yoksa yarm kalm bir backup 'n boyutu 0'dan farkl olduu iin BackOK
          // olmas iten bile deil !!!
          if FTempOrBackupFileSize>'0' then BackOK:=true else BackOK:=false;
        end;
      end;
    end else BackOK:=false;

    if BackOK=false then
    begin
      TX := TStringList.Create;
      try
        TX.LoadFromFile(GetTempPath+'~fibs_gbakout_'+FBackupNo+'.tmp');
        TX.Insert(0,'--------------------------------------------------------------------------------------   GBak.exe Output   ----------------------------------------------------------------------------------');
        TX.Add('---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------');
        for i:=0 to TX.Count-1 do TX[i]:='   '+TX[i];
        GOut:=TX.text;
      finally
        TX.Free;
      end;
      MailBody:=MailBody+#13#10+FTaskName+' has encountered Backup Error !! (1)';
    end else GOut:='';
    Windows.DeleteFile(PChar(GetTempPath+'~fibs_gbakout_'+FBackupNo+'.tmp'));

    CopyOK:=false;
    CopyOK2:=false;
    CopyOK3:=false;
    ZipOK:=false;
    if (BackOK and FCompressBackup) then // Backup tamam ise ve Zip steniyorsa
    begin
      Synchronize(RefreshPD10);
      try
        CompLevel:=6;
        if FCompDegree='None' then CompLevel:=0
        else if FCompDegree='Fastest' then CompLevel:=1
        else if FCompDegree='Default' then CompLevel:=6
        else if FCompDegree='Maximum' then CompLevel:=9;
        try
          FZippedBackupFile:=ChangeFileExt(FBackupFile,'.GZ');
          FZippedMirrorFile:=ChangeFileExt(FMirrorFile,'.GZ');
          FZippedMirrorFile2:=ChangeFileExt(FMirrorFile2,'.GZ');
          FZippedMirrorFile3:=ChangeFileExt(FMirrorFile3,'.GZ');

          ZipOK:=GZipExFunc(FTempBackFileName,FZippedBackupFile,StrToOEM(ExtractFileName(FBackupFile)),CompLevel,FErrorInfo)=0;
        except
          ZipOK:=false;
        end;
      finally

      end;
    end;

    if TempOK then DeleteFile(PChar(FTempBackFileName)); //Let's delete *.TMP y silelim
    // if task is running in auto mode then delete oldest backup file
    if FManAuto=true then
    begin
      //if new backup is created then delete oldest backup file
      if BackOK then
      begin
        FDeletedBU:=DeleteOldest(BUPath,'the Backup Dir');
      end;
    end;
    MirrorIsFtp:=false;
    if (BackOK and FDoMirror) then // Backup is OK and Mirroring onto #1 is wanted..
    begin
      Synchronize(RefreshPD20);
      MirrorPath:=ExtractFilePathEx(FMirrorFile);
      if (Lowercase(Copy(Mirrorpath,1,6))='ftp://') then MirrorIsFtp:=true else MirrorIsFtp:=false;
      if FCompressBackup then
      begin
        if ZipOK then CopyOK:=MirrorFileEx(FZippedBackupFile,FZippedMirrorFile,MFtpCopy,FConnectionType) else CopyOK:=false;
      end else CopyOK:=MirrorFileEx(FBackupFile,FMirrorFile,MFtpCopy,FConnectionType);
      // if task is running in auto mode then delete oldest file in the Mirror #1 directory.
      if FManAuto=true then
      begin
        // if mirroring onto Mirror #1 is OK then delete oldest file in the Mirror #1 directory.
        if CopyOK then
        begin
          FDeletedMR:=DeleteOldest(MirrorPath,'Mirror Dir #1');
        end;
      end;
    end;

    Mirror2IsFtp:=false;
    if (BackOK and FDoMirror2) then // Backup tamam ise ve Mirroring steniyorsa
    begin
      Synchronize(RefreshPD22);
      MirrorPath2:=ExtractFilePathEx(FMirrorFile2);
      if (Lowercase(Copy(Mirrorpath2,1,6))='ftp://') then Mirror2IsFtp:=true else Mirror2IsFtp:=false;
      if FCompressBackup then
      begin
        if ZipOK then CopyOK2:=MirrorFileEx(FZippedBackupFile,FZippedMirrorFile2,MFtpCopy2,FConnectionType) else CopyOK2:=false;
      end else CopyOK2:=MirrorFileEx(FBackupFile,FMirrorFile2,MFtpCopy2,FConnectionType);
      // if task is running in auto mode then delete oldest file in the Mirror #2 directory.
      if FManAuto=true then
      begin
        // if mirroring onto Mirror #2 is OK then delete oldest file in the Mirror #2 directory.
        if CopyOK2 then
        begin
          FDeletedMR2:=DeleteOldest(MirrorPath2,'Mirror Dir #2');
        end;
      end;
    end;

    Mirror3IsFtp:=false;
    if (BackOK and FDoMirror3) then // Backup tamam ise ve Mirroring steniyorsa
    begin
      Synchronize(RefreshPD23);
      MirrorPath3:=ExtractFilePathEx(FMirrorFile3);
      if (Lowercase(Copy(Mirrorpath3,1,6))='ftp://') then Mirror3IsFtp:=true else Mirror3IsFtp:=false;
      if FCompressBackup then
      begin
        if ZipOK then CopyOK3:=MirrorFileEx(FZippedBackupFile,FZippedMirrorFile3,MFtpCopy3,FConnectionType) else CopyOK3:=false;
      end else CopyOK3:=MirrorFileEx(FBackupFile,FMirrorFile3,MFtpCopy3,FConnectionType);
      // if task is running in auto mode then delete oldest file in the Mirror #3 directory.
      if FManAuto=true then
      begin
        // if mirroring onto Mirror #3 is OK then delete oldest file in the Mirror #3 directory.
        if CopyOK3 then
        begin
          FDeletedMR3:=DeleteOldest(MirrorPath3,'Mirror Dir #3');
        end;
      end;
    end;

    AEndTime:=Now;
    if AEndTime-AStartTime<1 then AEndTimeStr:=TimeToStr(AEndTime,FSettings) else AEndTimeStr:=DateTimeToStr(AEndTime,FSettings);
//  EMail   Synchronize(RefreshPD30);

    if FManAuto=False then MAuto:='MANUAL ' else MAuto:='';
    if BackOK then
    begin
      if FCompressBackup then // Zip istenmise
      begin
        if (ZipOK=false) then
        begin
          MBack:='ERROR !!  NEW BACKUP FILE ( '+FBackupFile+' ) COULDN''T BEEN ZIPPED !'#13#10+FErrorInfo+#13#10;
          MailBody:=MailBody+#13#10+FTaskName+' has encountered Backup Error !! (2)'#13#10+'          '+MBack;
        end else MBack:='New Backup File created as "'+FZippedBackupFile+'" on the Backup Dir.'#13#10;
      end else MBack:='New Backup File created as "'+FBackupFile+'" on the Backup Dir.'#13#10;
    end else
    begin
      if FCompressBackup then
      begin
        if TempOK=false
           then MBack:='ERROR !!  NEW BACKUP FILE ( '+FBackupFile+' ) COULDN''T BEEN CREATED Because of not creating Temporary File !'#13#10+'   GBak Params : >'+FFullGBakParams+'< '+#13#10
           else MBack:='ERROR !!  NEW BACKUP FILE ( '+FBackupFile+' ) COULDN''T BEEN CREATED !'#13#10+'   GBak Params : >'+FFullGBakParams+'< '+#13#10;
      end else MBack:='ERROR !!  NEW BACKUP FILE ( '+FBackupFile+' ) COULDN''T BEEN CREATED !'#13#10+'   GBak Params : >'+FFullGBakParams+'< '+#13#10;
      MailBody:=MailBody+#13#10+FTaskName+' has encountered Backup Error !! (3)'#13#10+'          '+MBack;
      MBack:=MBack+GOut;
    end;

    if BackOK then
    begin
      if FCompressBackup
      then MBack:=MBack+'   Backup FileSize (uncompressed size) = '+FTempOrBackupFileSize+' Byte '+#13#10
      else MBack:=MBack+'   Backup FileSize = '+FTempOrBackupFileSize+' Byte '+#13#10;
    end;

    if MirrorIsFtp then
    begin
      if FConnectionType=0 then
      begin
        FtpModeInfoA:=' in ACTIVE Mode.';
        FtpModeInfoB:=' (FTP Mode ACTIVE)';
      end else
      begin
        FtpModeInfoA:=' in PASSIVE Mode.';
        FtpModeInfoB:=' (FTP Mode PASSIVE)';
      end;
    end;

    if CopyOK then
    begin
      if FCompressBackup
        then MCopy:='New Backup File is mirrored onto Mirror #1 ( '+FZippedMirrorFile+' )'+FtpModeInfoA+#13#10
        else MCopy:='New Backup File is mirrored onto Mirror #1 ( '+FMirrorFile+' )'+FtpModeInfoA+#13#10;
    end else
    begin
      if FDoMirror=false then
      begin
        MCopy:='Mirroring #1 is disable..'#13#10
      end else
      begin
        if BackOK
        then MCopy:='ERROR !!  NEW BACKUP FILE COULDN''T BEEN MIRRORED ONTO MIRROR #1 ( '+MirrorPath+' ) !'+FtpModeInfoB+#13#10
        else MCopy:='ERROR !!  NEW BACKUP FILE COULDN''T BEEN MIRRORED ONTO MIRROR #1 ( No Backup File to Mirror !! ) !'+FtpModeInfoB+#13#10;
        MailBody:=MailBody+#13#10+FTaskName+' has encountered a Mirroring #1 Error !! (4)'#13#10+'          '+MCopy;
      end;
    end;

    if Mirror2IsFtp then
    begin
      if FConnectionType=0 then
      begin
        FtpModeInfoA:=' in ACTIVE Mode.';
        FtpModeInfoB:=' (FTP Mode ACTIVE)';
      end else
      begin
        FtpModeInfoA:=' in PASSIVE Mode.';
        FtpModeInfoB:=' (FTP Mode PASSIVE)';
      end;
    end;

    if CopyOK2 then
    begin
      if FCompressBackup
        then MCopy2:='New Backup File is mirrored onto Mirror #2 ( '+FZippedMirrorFile2+' )'+FtpModeInfoA+#13#10
        else MCopy2:='New Backup File is mirrored onto Mirror #2 ( '+FMirrorFile2+' )'+FtpModeInfoA+#13#10;
    end else
    begin
      if FDoMirror2=false then
      begin
        MCopy2:='Mirroring #2 is disable..'#13#10
      end else
      begin
         if BackOK
         then MCopy2:='ERROR !!  NEW BACKUP FILE COULDN''T BEEN MIRRORED ONTO MIRROR #2 ( '+MirrorPath2+' ) !'+FtpModeInfoB+#13#10
         else MCopy2:='ERROR !!  NEW BACKUP FILE COULDN''T BEEN MIRRORED ONTO MIRROR #2 ( No Backup File to Mirror !! ) !'+FtpModeInfoB+#13#10;
         MailBody:=MailBody+#13#10+FTaskName+' has encountered a Mirroring #2 Error !! (5)'#13#10+'          '+MCopy;
      end;
    end;

    if Mirror3IsFtp then
    begin
      if FConnectionType=0 then
      begin
        FtpModeInfoA:=' in ACTIVE Mode.';
        FtpModeInfoB:=' (FTP Mode ACTIVE)';
      end else
      begin
        FtpModeInfoA:=' in PASSIVE Mode.';
        FtpModeInfoB:=' (FTP Mode PASSIVE)';
      end;
    end;

    if CopyOK3 then
    begin
      if FCompressBackup
         then MCopy3:='New Backup File is mirrored onto Mirror #3 ( '+FZippedMirrorFile3+' )'+FtpModeInfoA+#13#10
         else MCopy3:='New Backup File is mirrored onto Mirror #3 ( '+FMirrorFile3+' )'+FtpModeInfoA+#13#10;
    end else
    begin
      if FDoMirror3=false then
      begin
        MCopy3:='Mirroring #3 is disable..'#13#10
      end else
      begin
        if BackOK
         then MCopy2:='ERROR !!  NEW BACKUP FILE COULDN''T BEEN MIRRORED ONTO MIRROR #3 ( '+MirrorPath3+' ) !'+FtpModeInfoB+#13#10
         else MCopy2:='ERROR !!  NEW BACKUP FILE COULDN''T BEEN MIRRORED ONTO MIRROR #3 ( No Backup File to Mirror !! ) !'+FtpModeInfoB+#13#10;
         MailBody:=MailBody+#13#10+FTaskName+' has encountered a Mirroring #3 Error !! (6)'#13#10+'          '+MCopy;
      end;
    end;
    MDel:='';
    Synchronize(RefreshPD30);
    if FManAuto=true then
    begin
      MDel:=FDeletedBU+FDeletedMR+FDeletedMR2+FDeletedMR3;
    end else MDel:='None of out-of-criteria backup file(s) on backup and/or mirrors dirs has been deleted because of being Manual backup Mode'#13#10;

    if trim(FExternalFile)='' then
    begin
      MBatch:='External file execution is Disabled.'#13#10;
    end else
    begin
      Synchronize(RefreshPD31);
      if FCompressBackup then ParamsToBatch:=FZippedBackupFile else ParamsToBatch:=FBackupFile;
      ParamsToBatch:='"'+ParamsToBatch+'"';
      if FUseParams=true then
      begin
        if FShowExternalWin=true then
        begin
          ExternalResultCode:=ShellExecute(handle,'open',PChar(FExternalFile),PChar(ParamsToBatch),nil,SW_SHOWNORMAL);
          BatchFileOk:=ExternalResultCode>32
        end else
        begin
          ExternalResultCode:=ShellExecute(handle,'open',PChar(FExternalFile),PChar(ParamsToBatch),nil,SW_HIDE);
          BatchFileOk:=ExternalResultCode>32
        end;
      end else
      begin
        if FShowExternalWin=true then
        begin
          ExternalResultCode:=ShellExecute(handle,'open',PChar(FExternalFile),nil,nil,SW_SHOWNORMAL);
          BatchFileOk:=ExternalResultCode>32
        end else
        begin
          ExternalResultCode:=ShellExecute(handle,'open',PChar(FExternalFile),nil,nil,SW_HIDE);
          BatchFileOk:=ExternalResultCode>32
        end;
      end;
      if BatchFileOk=false then
      begin
        if FUseParams=true
          then MBatch:='ERROR !!  EXTERNAL FILE '+FExternalFile+' COULDN''T BEEN RUNNED WITH PARAM '+ParamsToBatch+' !! (Error Code ='+IntToStr(ExternalResultCode)+') '+#13#10
          else MBatch:='ERROR !!  EXTERNAL FILE '+FExternalFile+' COULDN''T BEEN RUNNED !! (Error Code ='+IntToStr(ExternalResultCode)+') '+#13#10;
      end else MBatch:='External File "'+FExternalFile+'" has been started to run with parameter '+ParamsToBatch+' at '+DateTimeToStr(Now)+#13#10
    end;

    if((ValidateOK=false)or
       (BackOK=false)or
       ((FCompressBackup=true)and(ZipOK=false))or
       ((FDoMirror=true)and(CopyOK=false))or
       ((FDoMirror2=true)and(CopyOK2=false))or
       ((FDoMirror3=true)and(CopyOK3=false)))then
    begin
      // There is a problem.
      if trim(FMailTo)<>''then
      begin
        // Wanted EMail Notification
        Synchronize(RefreshPD13);
        EMailOK:=CreateEmail(FMailTo,'','Backup Error Warning for '+FTaskName+' Backup # '+FBackupNo,MailBody,'');
        if EMailOK then MMail:='Warning EMail sended to '+FMailTo+' at '+DateTimeToStr(Now)+#13#10
                   else MMail:='ERROR !!  WARNING EMAIL COULDN''T BEEN SENDED TO "'+FMailTo+'"  !!. '+DateTimeToStr(Now)+'   (Smtp Server Error Info :'+FErrorInfo+')'+#13#10;
      end else MMail:='EMail Notification is Disabled. So, no warning mail has been sended..'#13#10;
    end else MMail:='EMail Notification is enabled. (No email notification has been sended because there is noting to notify.)'#13#10;

    BackupInfoStr:=#13#10+
                 ' '+MAuto+'BACKUP TASK " '+FTaskName+'-'+DateTimeToStr(FAlarm,FSettings)+' "'#13#10+
                 '   Executed between '+DateTimeToStr(AStartTime,FSettings)+' and '+AEndTimeStr+ ' with " '+FBackupOptions+' " Backup Options.'#13#10+
//               '   Gbak Params    : '+FGBakParams+#13#10+
                 '   Database File  : '+FDatabaseFile+#13#10+
                 '   '+GFixOut+
                 '   '+MBack+
                 '   '+MCopy+MFtpCopy+
                 '   '+MCopy2+MFtpCopy2+
                 '   '+MCopy3+MFtpCopy3+
                 '   '+MDel+
                 '   '+MBatch+
                 '   '+MMail;
    Synchronize(RefreshPD40);
    WriteLog(BackupInfoStr);
  finally
    Synchronize(HidePD);
  end;
end;

// There is one more StrToOEM  function in the JRZip Unit
function TBackup.StrToOEM (s : string) : string;
var
  i  : integer;
begin
  for i:=1 to length(s) do
  begin
    if s[i]>=#128 then s[i]:='_';
  end;
  result:=s;
end;


end.
