
unit RetMonitorTools;
//Author : retnyg @ http://krazz.net/retnyg

interface
uses Windows,Types,Graphics;

const
 MinFact = 125;
 MaxFact = 167;
 MONITOR_DEFAULTTONULL = $0;    //If the monitor is not found, return 0
 MONITOR_DEFAULTTOPRIMARY = $1; //If the monitor is not found, return the primary monitor
 MONITOR_DEFAULTTONEAREST = $2; //If the monitor is not found, return the nearest monitor

type

  PMONITORINFO = ^TMONITORINFO;
  TMONITORINFO = record
    cbSize:DWord;
    rcMonitor:TRect;
    rcWork:TRect;
    dwFlags:DWord;
  end;

// own functions
 function GetDesktopDimensions:Trect;
 function GuessNumberMonitors:byte;
 function GetNumberMonitors:byte;
 function MoveWindowToMonitor(hnd: HWND; monNum:byte):boolean;
 function GetMonitorFromWindow(hnd: HWND):byte;

// imported API functions
 function GetMonitorInfo(AMonitorHandle: pointer; Var ADataRecord: TMonitorInfo): Boolean;
          stdcall; External 'User32.dll' Name 'GetMonitorInfoA'; overload;
 function MonitorFromPoint(APoint: TPoint; AFlags: DWORD): pointer;
          stdcall; External 'User32.dll';
 function MonitorFromWindow(AWindowHandle: HWND; AFlags: DWORD): pointer;
          stdcall; External 'User32.dll';

implementation

function GetDesktopDimensions:Trect;
var
  DC: HDC;
  Cn: TCanvas;
begin
  DC:=GetDC(0);
  Cn:=TCanvas.Create;
  cn.Handle:=DC;
  Result:=cn.ClipRect;
  cn.Free;
  ReleaseDc(0,DC);
end;

function GuessNumberMonitors:byte;
var
  dr: TRect;
begin
  result := 1;
  dr:=GetDesktopDimensions;
  while (dr.Right * 100) div dr.bottom > maxFact do
  begin
    inc(result);
    dr.Right := (dr.Right) - ((dr.bottom * minFact) div 100);
  end;
end;

function GetNumberMonitors:byte;
// slower than GuessNumberMonitors
var
  LastMon, NewMon: Pointer;
  dr:TRect;
begin
  Result := 0;
  LastMon:=nil;
  dr:=GetDesktopDimensions;
  while dr.right > 0 do
  begin
    NewMon := MonitorFromPoint(dr.BottomRight,MONITOR_DEFAULTTONEAREST);
    if NewMon <> LastMon then
    begin
      inc(result);
      LastMon := NewMon;
    end;
    dec(dr.Right,100);
  end;
end;

function moveWindowToMonitor(hnd: HWND; MonNum:byte):boolean;
var
  dr: TRect;
  NumMon:Byte;
  PxpMon,NewX:DWord;
begin
  Result:=false;
  NumMon:=GuessNumberMonitors;
  if (NumMon > 0) and (MonNum > 0) then
  begin
    dr:=GetDesktopDimensions;
    PxpMon:=dr.right div NumMon;
    if GetwindowRect(hnd,dr) then
    begin
      NewX:=(dr.Left mod PxpMon)+((PxpMon*MonNum)-PxpMon);
      if MoveWindow(hnd, NewX, dr.Top, dr.Right - dr.Left, dr.Bottom - dr.Top,true) then
      Result := true;
    end;
  end;
end;

function GetMonitorFromWindow(hnd: HWND):byte;
var
  dr: TRect;
  NumMon: Byte;
  PxpMon: DWord;
begin
  Result:=1;
  NumMon:=GuessNumberMonitors;
  if (NumMon > 1) then
  begin
    dr:=GetDesktopDimensions;
    PxpMon:=dr.Right div NumMon;
    if GetWindowRect(hnd,dr) then Result := (dr.Left div PxpMon) + 1;
  end;
end;


end.
